<?php
    
    namespace artbox\catalog\helpers;
    
    use artbox\catalog\models\Brand;
    use artbox\catalog\models\BrandLang;
    use artbox\catalog\models\Category;
    use artbox\catalog\models\Filter;
    use artbox\catalog\models\Product;
    use artbox\catalog\models\ProductOptionCompl;
    use artbox\catalog\models\ProductOptionComplLang;
    use artbox\catalog\models\ProductOptionExcl;
    use artbox\catalog\models\ProductOptionExclLang;
    use artbox\catalog\models\VariantOptionCompl;
    use artbox\catalog\models\VariantOptionComplLang;
    use artbox\catalog\models\VariantOptionExcl;
    use artbox\catalog\models\VariantOptionExclLang;
    use artbox\core\models\Alias;
    use yii\base\InvalidConfigException;
    use yii\base\Object;
    use yii\db\ActiveQuery;
    use yii\db\ActiveRecord;
    use yii\helpers\ArrayHelper;
    use yii\helpers\Json;
    
    /**
     * Class FilterHelper able to work with filters: building query for products and generate filter links.
     */
    class FilterHelper extends Object
    {
        /**
         * Object with stored rewrote filters and methods to replace theme
         *
         * @var Filter
         */
        public $filterObj;
        
        /**
         * Whether filter is loaded
         *
         * @var bool
         */
        protected $loaded = false;
        
        /**
         * Filter string value
         *
         * @var string
         */
        protected $filter = '';
        
        /**
         * Array of active aliases
         *
         * @var array
         */
        protected $activeAliases = [];
        
        /**
         * Filter ids grouped by model classnames
         *
         * @var array
         */
        protected $groupFilters = [];
        
        /**
         * Filter models grouped by model classnames
         *
         * @var array
         */
        protected $groupModels = [];
    
        /**
         * @inheritdoc
         */
        public function init()
        {
            $this->filterObj = \Yii::createObject(Filter::className());
        }

        /**
         * return array of objects
         * @return array
         */
        public function getActiveOptions(): array
        {
            $activeOptions = [];
            if(!empty($this->activeAliases)) {
                foreach ($this->activeAliases as $activeAlias) {
                    $option = new \stdClass();
                    $option->title = $activeAlias->title;
                    $activeOptions[] = $option;
                }
            }
            return $activeOptions;
        }

        /**
         * Getter for $groupModels
         *
         * @return array
         */
        public function getGroupModels(): array
        {
            return $this->groupModels;
        }
        
        /**
         * Getter for $loaded
         *
         * @return bool
         */
        public function getLoaded(): bool
        {
            return $this->loaded;
        }
        
        /**
         * Set filter value and makes further actions, can only be performed once
         *
         * @param string $filter
         *
         * @throws \yii\base\InvalidConfigException
         */
        public function setFilter(string $filter)
        {
            if (empty($this->filter)) {
                $this->loaded = true;
                $this->filter = $filter;
                $this->groupModels($this->groupFilters($filter));
                $this->getPrices($filter);
            } else {
                throw new InvalidConfigException(\Yii::t('catalog', 'Filter can be initiated only once'));
            }
        }
        
        /**
         * Getter for $filter
         *
         * @return string
         */
        public function getFilter(): string
        {
            return $this->filter;
        }
        
        /**
         * Getter for $groupFilters
         *
         * @return array
         */
        public function getGroupFilters(): array
        {
            return $this->groupFilters;
        }
        
        public function getActiveAliases(): array
        {
            return $this->activeAliases;
        }
        
        /**
         * Explode filter string into filter array
         *
         * @param string $filter
         *
         * @return array
         */
        protected function parseFilter(string $filter)
        {
            return explode('_', $filter);
        }
        
        /**
         * Get array of aliases for filter
         *
         * @param string $filter
         *
         * @return Alias[]
         */
        protected function getAliases(string $filter): array
        {
            $filters = $this->parseFilter($filter);
            $aliases = Alias::find()
                            ->where([ 'value' => $filters ])
                            ->all();
            $this->activeAliases = $aliases;
            return $aliases;
        }
        
        /**
         * Group filter ids by classname
         *
         * @param string $filter
         *
         * @return array
         */
        protected function groupFilters(string $filter)
        {
            $aliases = $this->getAliases($filter);
            $result = [];
            foreach ($aliases as $alias) {
                if (in_array(
                    $alias->entity,
                    [
                        BrandLang::className(),
                        ProductOptionComplLang::className(),
                        ProductOptionExclLang::className(),
                        VariantOptionExclLang::className(),
                        VariantOptionComplLang::className(),
                    ]
                )) {
                    $id = $this->getIdFromRoute($alias->route);
                    if ($id) {
                        if (preg_match("/^([\w-\\\\]*\\\\\w+)Lang$/", $alias->entity, $matches)) {
                            $result[ $matches[ 1 ] ][] = $id;
                        } else {
                            $result[ $alias->entity ][] = $id;
                        }
                    }
                }
            }
            $this->groupFilters = $result;
            return $result;
        }
        
        /**
         * Group filter models by classname.
         *
         * @param array $filter
         *
         * @return array
         */
        protected function groupModels(array $filter)
        {
            $result = [];
            foreach ($filter as $model => $ids) {
                switch ($model) {
                    case Brand::className():
                        $result[ $model ] = Brand::find()
                                                 ->where([ 'id' => $ids ])
                                                 ->joinWith('lang', false)
                                                 ->orderBy(
                                                     [
                                                         'brand.sort' => SORT_ASC,
                                                         'brand_lang.title' => SORT_ASC,
                                                     ]
                                                 )
                                                 ->all();
                        break;
                    case ProductOptionCompl::className():
                        $result[ $model ] = ProductOptionCompl::find()
                                                              ->where([ 'product_option_compl.id' => $ids ])
                                                              ->joinWith('lang.alias', true)
                                                              ->joinWith('group', true)
                                                              ->orderBy(
                                                                  [
                                                                      'product_option_group_compl.sort' => SORT_ASC,
                                                                      'product_option_group_compl.id'   => SORT_ASC,
                                                                      'product_option_compl.sort'       => SORT_ASC,
                                                                      'product_option_compl_lang.value' => SORT_ASC,
                            
                                                                  ]
                                                              )
                                                              ->all();
                        break;
                    case ProductOptionExcl::className():
                        $result[ $model ] = ProductOptionExcl::find()
                                                             ->where([ 'product_option_excl.id' => $ids ])
                                                             ->joinWith('lang.alias', true)
                                                             ->joinWith('group', true)
                                                             ->orderBy(
                                                                 [
                                                                     'product_option_group_excl.sort' => SORT_ASC,
                                                                     'product_option_group_excl.id'   => SORT_ASC,
                                                                     'product_option_excl.sort'       => SORT_ASC,
                                                                     'product_option_excl_lang.value' => SORT_ASC,
                            
                                                                 ]
                                                             )
                                                             ->all();
                        break;
                    case VariantOptionCompl::className():
                        $result[ $model ] = VariantOptionCompl::find()
                                                              ->where([ 'variant_option_compl.id' => $ids ])
                                                              ->joinWith('lang.alias', true)
                                                              ->joinWith('group', true)
                                                              ->orderBy(
                                                                  [
                                                                      'variant_option_group_compl.sort' => SORT_ASC,
                                                                      'variant_option_group_compl.id'   => SORT_ASC,
                                                                      'variant_option_compl.sort'       => SORT_ASC,
                                                                      'variant_option_compl_lang.value' => SORT_ASC,
                            
                                                                  ]
                                                              )
                                                              ->all();
                        break;
                    case VariantOptionExcl::className():
                        $result[ $model ] = VariantOptionExcl::find()
                                                             ->where([ 'variant_option_excl.id' => $ids ])
                                                             ->joinWith('lang.alias', true)
                                                             ->joinWith('group', true)
                                                             ->orderBy(
                                                                 [
                                                                     'variant_option_group_excl.sort' => SORT_ASC,
                                                                     'variant_option_group_excl.id'   => SORT_ASC,
                                                                     'variant_option_excl.sort'       => SORT_ASC,
                                                                     'variant_option_excl_lang.value' => SORT_ASC,
                            
                                                                 ]
                                                             )
                                                             ->all();
                        break;
                }
            }
            $this->groupModels = $result;
            return $result;
        }
        
        /**
         * Get Id from Alias route column
         *
         * @param string $json
         *
         * @return int|null
         */
        public function getIdFromRoute(string $json)
        {
            $route = Json::decode($json);
            return empty($route[ 'id' ]) ? null : (int) $route[ 'id' ];
        }
        
        /**
         * Build query to retrieve Products according to current filter
         *
         * @param string|null $productClass
         * @param bool        $eagerload
         *
         * @return \yii\db\ActiveQuery
         */
        public function buildQuery(string $productClass = null, bool $eagerload = false): ActiveQuery
        {
            if (!$productClass) {
                $productClass = Product::className();
            }
            $groups = $this->groupFilters;
            $query = call_user_func(
                [
                    $productClass,
                    'find',
                ]
            );
            foreach ($groups as $group => $ids) {
                switch ($group) {
                    case Brand::className():
                        $this->buildBrand($ids, $query);
                        break;
                    case ProductOptionCompl::className():
                        $this->buildProductCompl($ids, $query, $eagerload);
                        break;
                    case ProductOptionExcl::className():
                        $this->buildProductExcl($ids, $query, $eagerload);
                        break;
                    case VariantOptionCompl::className():
                        $this->buildVariantCompl($ids, $query, $eagerload);
                        break;
                    case VariantOptionExcl::className():
                        $this->buildVariantExcl($ids, $query, $eagerload);
                        break;
                    case 'prices':
                        $this->buildPrices($ids, $query, $eagerload);
                        break;
                }
            }
            return $query;
        }
        
        /**
         * Get brands for provided Category
         *
         * @param \artbox\catalog\models\Category $category
         *
         * @return array
         */
        public function getBrands(Category $category): array
        {
            $brands = [];
            foreach ($category->products as $product) {
                /**
                 * @var Product $product
                 */
                if (!empty($product->brand_id) && !array_key_exists($product->brand_id, $brands)) {
                    $brands[ $product->brand_id ] = $product->brand;
                }
            }
            return $brands;
        }
        
        protected function buildBrand(array $ids, ActiveQuery $query)
        {
            $query->andWhere([ 'product.brand_id' => $ids ]);
        }
        
        protected function buildProductCompl(array $ids, ActiveQuery $query, bool $eagerload = false)
        {
            foreach ($ids as $id) {
                $query->innerJoinWith([ "productToProductOptionCompls poc$id" ], $eagerload)
                      ->andWhere([ "poc$id.product_option_compl_id" => $id ]);
            }
        }
        
        protected function buildProductExcl(array $ids, ActiveQuery $query, bool $eagerload = false)
        {
            $query->innerJoinWith('productToProductOptionExcls', $eagerload)
                  ->andWhere([ 'product_to_product_option_excl.product_option_excl_id' => $ids ]);
        }
        
        protected function buildVariantCompl(array $ids, ActiveQuery $query, bool $eagerload = false)
        {
            foreach ($ids as $id) {
                $query->innerJoinWith("variants.variantToVariantOptionCompls voc$id", $eagerload)
                      ->andWhere([ "voc$id.variant_option_compl_id" => $ids ]);
            }
        }
        
        protected function buildVariantExcl(array $ids, ActiveQuery $query, bool $eagerload = false)
        {
            $query->innerJoinWith('variants.variantToVariantOptionExcls', $eagerload)
                  ->andWhere([ 'variant_to_variant_option_excl.variant_option_excl_id' => $ids ]);
        }
        
        protected function buildPrices(array $ids, ActiveQuery $query, bool $eagerload = false)
        {
            $query->innerJoinWith('variants', $eagerload)
                  ->andWhere(
                      [
                          'between',
                          'variant.price',
                          (float) $ids[ 0 ],
                          (float) $ids[ 1 ],
                      ]
                  );
        }
        
        /**
         * Build link for filter including or excluding (if already set) provided $model
         *
         * @param \yii\db\ActiveRecord $model
         *
         * @return string
         */
        public function buildLink(ActiveRecord $model = null)
        {
            $groups = $this->groupModels;
            $groupsFilter = $this->groupFilters;
            $link = '';
            $skip = false;
            if (!empty($model)) {
                switch ($model::className()) {
                    case Brand::className():
                        if (!empty($groups[ Brand::className() ])) {
                            foreach ($groups[ Brand::className() ] as $index => $object) {
                                /**
                                 * @var Brand $object
                                 * @var Brand $model
                                 */
                                if ($object->lang->alias->value == $model->lang->alias->value) {
                                    unset($groups[ Brand::className() ][ $index ]);
                                    $skip = true;
                                    break;
                                }
                            }
                        }
                        if (!$skip) {
                            $groups[ Brand::className() ][] = $model;
                            usort(
                                $groups[ Brand::className() ],
                                function ($a, $b) {
                                    /**
                                     * @var Brand $a
                                     * @var Brand $b
                                     */
                                    if ($a->sort > $b->sort) {
                                        return 1;
                                    } elseif ($a->sort < $b->sort) {
                                        return -1;
                                    } else {
                                        return strcmp($a->lang->title, $b->lang->title);
                                    }
                                }
                            );
                            $link = implode(
                                '_',
                                ArrayHelper::getColumn($groups[ Brand::className() ], 'lang.alias.value')
                            );
                        }
                        if (!empty($groups[ ProductOptionCompl::className() ])) {
                            if (empty($link)) {
                                $link = $this->writePOC($groups);
                            } else {
                                $link .= '_' . $this->writePOC($groups);
                            }
                        }
                        if (!empty($groups[ ProductOptionExcl::className() ])) {
                            if (empty($link)) {
                                $link = $this->writePOE($groups);
                            } else {
                                $link .= '_' . $this->writePOE($groups);
                            }
                        }
                        if (!empty($groups[ VariantOptionCompl::className() ])) {
                            if (empty($link)) {
                                $link = $this->writeVOC($groups);
                            } else {
                                $link .= '_' . $this->writeVOC($groups);
                            }
                        }
                        if (!empty($groups[ VariantOptionExcl::className() ])) {
                            if (empty($link)) {
                                $link = $this->writeVOE($groups);
                            } else {
                                $link .= '_' . $this->writeVOE($groups);
                            }
                        }
                        break;
                    case ProductOptionCompl::className():
                        if (!empty($groups[ ProductOptionCompl::className() ])) {
                            foreach ($groups[ ProductOptionCompl::className() ] as $index => $object) {
                                /**
                                 * @var ProductOptionCompl $object
                                 * @var ProductOptionCompl $model
                                 */
                                if ($object->lang->alias->value == $model->lang->alias->value) {
                                    unset($groups[ ProductOptionCompl::className() ][ $index ]);
                                    $skip = true;
                                    break;
                                }
                            }
                        }
                        if (!$skip) {
                            $groups[ ProductOptionCompl::className() ][] = $model;
                            usort(
                                $groups[ ProductOptionCompl::className() ],
                                function ($a, $b) {
                                    /**
                                     * @var ProductOptionCompl $a
                                     * @var ProductOptionCompl $b
                                     */
                                    if ($a->group->sort > $b->group->sort) {
                                        return 1;
                                    } elseif ($a->group->sort < $b->group->sort) {
                                        return -1;
                                    } else {
                                        if ($a->group->id > $b->group->id) {
                                            return 1;
                                        } elseif ($a->group->id < $b->group->id) {
                                            return -1;
                                        } else {
                                            if ($a->sort > $b->sort) {
                                                return 1;
                                            } elseif ($a->sort < $b->sort) {
                                                return -1;
                                            } else {
                                                return strcasecmp($a->lang->value, $b->lang->value);
                                            }
                                        }
                                    }
                                }
                            );
                        }
                        if (!empty($groups[ Brand::className() ])) {
                            $link = $this->writeBrands($groups);
                        }
                        if (empty($link)) {
                            $link = implode(
                                '_',
                                ArrayHelper::getColumn(
                                    $groups[ ProductOptionCompl::className() ],
                                    'lang.alias.value'
                                )
                            );
                        } else {
                            if (!empty(
                            ArrayHelper::getColumn(
                                $groups[ ProductOptionCompl::className() ],
                                'lang.alias.value'
                            )
                            )
                            ) {
                                $link .= '_' . implode(
                                        '_',
                                        ArrayHelper::getColumn(
                                            $groups[ ProductOptionCompl::className() ],
                                            'lang.alias.value'
                                        )
                                    );
                            }
                        }
                        if (!empty($groups[ ProductOptionExcl::className() ])) {
                            $link .= '_' . $this->writePOE($groups);
                        }
                        if (!empty($groups[ VariantOptionCompl::className() ])) {
                            $link .= '_' . $this->writeVOC($groups);
                        }
                        if (!empty($groups[ VariantOptionExcl::className() ])) {
                            $link .= '_' . $this->writeVOE($groups);
                        }
                        break;
                    case ProductOptionExcl::className():
                        if (!empty($groups[ ProductOptionExcl::className() ])) {
                            foreach ($groups[ ProductOptionExcl::className() ] as $index => $object) {
                                /**
                                 * @var ProductOptionExcl $object
                                 * @var ProductOptionExcl $model
                                 */
                                if ($object->lang->alias->value == $model->lang->alias->value) {
                                    unset($groups[ ProductOptionExcl::className() ][ $index ]);
                                    $skip = true;
                                    break;
                                }
                            }
                        }
                        if (!$skip) {
                            $groups[ ProductOptionExcl::className() ][] = $model;
                            usort(
                                $groups[ ProductOptionExcl::className() ],
                                function ($a, $b) {
                                    /**
                                     * @var ProductOptionExcl $a
                                     * @var ProductOptionExcl $b
                                     */
                                    if ($a->group->sort > $b->group->sort) {
                                        return 1;
                                    } elseif ($a->group->sort < $b->group->sort) {
                                        return -1;
                                    } else {
                                        if ($a->group->id > $b->group->id) {
                                            return 1;
                                        } elseif ($a->group->id < $b->group->id) {
                                            return -1;
                                        } else {
                                            if ($a->sort > $b->sort) {
                                                return 1;
                                            } elseif ($a->sort < $b->sort) {
                                                return -1;
                                            } else {
                                                return strcasecmp($a->lang->value, $b->lang->value);
                                            }
                                        }
                                    }
                                }
                            );
                        }
                        if (!empty($groups[ Brand::className() ])) {
                            $link = $this->writeBrands($groups);
                        }
                        if (!empty($groups[ ProductOptionCompl::className() ])) {
                            if (!empty($link)) {
                                $link .= '_' . $this->writePOC($groups);
                            } else {
                                $link = $this->writePOC($groups);
                            }
                        }
                        if (empty($link)) {
                            $link = implode(
                                '_',
                                ArrayHelper::getColumn(
                                    $groups[ ProductOptionExcl::className() ],
                                    'lang.alias.value'
                                )
                            );
                        } else {
                            if (!empty(
                            ArrayHelper::getColumn(
                                $groups[ ProductOptionExcl::className() ],
                                'lang.alias.value'
                            )
                            )
                            ) {
                                $link .= '_' . implode(
                                        '_',
                                        ArrayHelper::getColumn(
                                            $groups[ ProductOptionExcl::className() ],
                                            'lang.alias.value'
                                        )
                                    );
                            }
                        }
                        if (!empty($groups[ VariantOptionCompl::className() ])) {
                            if (empty($link)) {
                                $link = $this->writeVOC($groups);
                            } else {
                                $link .= '_' . $this->writeVOC($groups);
                            }
                        }
                        if (!empty($groups[ VariantOptionExcl::className() ])) {
                            if (empty($link)) {
                                $link = $this->writeVOE($groups);
                            } else {
                                $link .= '_' . $this->writeVOE($groups);
                            }
                        }
                        break;
                    case VariantOptionCompl::className():
                        if (!empty($groups[ VariantOptionCompl::className() ])) {
                            foreach ($groups[ VariantOptionCompl::className() ] as $index => $object) {
                                /**
                                 * @var VariantOptionCompl $object
                                 * @var VariantOptionCompl $model
                                 */
                                if ($object->lang->alias->value == $model->lang->alias->value) {
                                    unset($groups[ VariantOptionCompl::className() ][ $index ]);
                                    $skip = true;
                                    break;
                                }
                            }
                        }
                        if (!$skip) {
                            $groups[ VariantOptionCompl::className() ][] = $model;
                            usort(
                                $groups[ VariantOptionCompl::className() ],
                                function ($a, $b) {
                                    /**
                                     * @var VariantOptionCompl $a
                                     * @var VariantOptionCompl $b
                                     */
                                    if ($a->group->sort > $b->group->sort) {
                                        return 1;
                                    } elseif ($a->group->sort < $b->group->sort) {
                                        return -1;
                                    } else {
                                        if ($a->group->id > $b->group->id) {
                                            return 1;
                                        } elseif ($a->group->id < $b->group->id) {
                                            return -1;
                                        } else {
                                            if ($a->sort > $b->sort) {
                                                return 1;
                                            } elseif ($a->sort < $b->sort) {
                                                return -1;
                                            } else {
                                                return strcasecmp($a->lang->value, $b->lang->value);
                                            }
                                        }
                                    }
                                }
                            );
                        }
                        if (!empty($groups[ Brand::className() ])) {
                            $link = $this->writeBrands($groups);
                        }
                        if (!empty($groups[ ProductOptionCompl::className() ])) {
                            if (!empty($link)) {
                                $link .= '_' . $this->writePOC($groups);
                            } else {
                                $link = $this->writePOC($groups);
                            }
                        }
                        if (!empty($groups[ ProductOptionExcl::className() ])) {
                            if (!empty($link)) {
                                $link .= '_' . $this->writePOE($groups);
                            } else {
                                $link = $this->writePOE($groups);
                            }
                        }
                        if (empty($link)) {
                            $link = implode(
                                '_',
                                ArrayHelper::getColumn(
                                    $groups[ VariantOptionCompl::className() ],
                                    'lang.alias.value'
                                )
                            );
                        } else {
                            if (!empty(
                            ArrayHelper::getColumn(
                                $groups[ VariantOptionCompl::className() ],
                                'lang.alias.value'
                            )
                            )
                            ) {
                                $link .= '_' . implode(
                                        '_',
                                        ArrayHelper::getColumn(
                                            $groups[ VariantOptionCompl::className() ],
                                            'lang.alias.value'
                                        )
                                    );
                            }
                        }
                        if (!empty($groups[ VariantOptionExcl::className() ])) {
                            if (empty($link)) {
                                $link = $this->writeVOC($groups);
                            } else {
                                $link .= '_' . $this->writeVOE($groups);
                            }
                        }
                        break;
                    case VariantOptionExcl::className():
                        if (!empty($groups[ VariantOptionExcl::className() ])) {
                            foreach ($groups[ VariantOptionExcl::className() ] as $index => $object) {
                                /**
                                 * @var VariantOptionExcl $object
                                 * @var VariantOptionExcl $model
                                 */
                                if ($object->lang->alias->value == $model->lang->alias->value) {
                                    unset($groups[ VariantOptionExcl::className() ][ $index ]);
                                    $skip = true;
                                    break;
                                }
                            }
                        }
                        if (!$skip) {
                            $groups[ VariantOptionExcl::className() ][] = $model;
                            usort(
                                $groups[ VariantOptionExcl::className() ],
                                function ($a, $b) {
                                    /**
                                     * @var VariantOptionExcl $a
                                     * @var VariantOptionExcl $b
                                     */
                                    if ($a->group->sort > $b->group->sort) {
                                        return 1;
                                    } elseif ($a->group->sort < $b->group->sort) {
                                        return -1;
                                    } else {
                                        if ($a->group->id > $b->group->id) {
                                            return 1;
                                        } elseif ($a->group->id < $b->group->id) {
                                            return -1;
                                        } else {
                                            if ($a->sort > $b->sort) {
                                                return 1;
                                            } elseif ($a->sort < $b->sort) {
                                                return -1;
                                            } else {
                                                return strcasecmp($a->lang->value, $b->lang->value);
                                            }
                                        }
                                    }
                                }
                            );
                        }
                        if (!empty($groups[ Brand::className() ])) {
                            $link = $this->writeBrands($groups);
                        }
                        if (!empty($groups[ ProductOptionCompl::className() ])) {
                            if (!empty($link)) {
                                $link .= '_' . $this->writePOC($groups);
                            } else {
                                $link = $this->writePOC($groups);
                            }
                        }
                        if (!empty($groups[ ProductOptionExcl::className() ])) {
                            if (!empty($link)) {
                                $link .= '_' . $this->writePOE($groups);
                            } else {
                                $link = $this->writePOE($groups);
                            }
                        }
                        if (!empty($groups[ VariantOptionCompl::className() ])) {
                            if (!empty($link)) {
                                $link .= '_' . $this->writeVOC($groups);
                            } else {
                                $link = $this->writeVOC($groups);
                            }
                        }
                        if (empty($link)) {
                            $link = implode(
                                '_',
                                ArrayHelper::getColumn(
                                    $groups[ VariantOptionExcl::className() ],
                                    'lang.alias.value'
                                )
                            );
                        } else {
                            if (!empty(
                            ArrayHelper::getColumn(
                                $groups[ VariantOptionExcl::className() ],
                                'lang.alias.value'
                            )
                            )
                            ) {
                                $link .= '_' . implode(
                                        '_',
                                        ArrayHelper::getColumn(
                                            $groups[ VariantOptionExcl::className() ],
                                            'lang.alias.value'
                                        )
                                    );
                            }
                        }
                        break;
                }
            } else {
                if (!empty($groups[ Brand::className() ])) {
                    $link = $this->writeBrands($groups);
                }
                if (!empty($groups[ ProductOptionCompl::className() ])) {
                    if (empty($link)) {
                        $link = $this->writePOC($groups);
                    } else {
                        $link .= '_' . $this->writePOC($groups);
                    }
                }
                if (!empty($groups[ ProductOptionExcl::className() ])) {
                    if (empty($link)) {
                        $link = $this->writePOE($groups);
                    } else {
                        $link .= '_' . $this->writePOE($groups);
                    }
                }
                if (!empty($groups[ VariantOptionCompl::className() ])) {
                    if (empty($link)) {
                        $link = $this->writeVOC($groups);
                    } else {
                        $link .= '_' . $this->writeVOC($groups);
                    }
                }
                if (!empty($groups[ VariantOptionExcl::className() ])) {
                    if (empty($link)) {
                        $link = $this->writeVOE($groups);
                    } else {
                        $link .= '_' . $this->writeVOE($groups);
                    }
                }
            }
            if (!empty($groupsFilter[ 'prices' ])) {
                if (!empty($link)) {
                    $link .= '_' . 'price-' . $groupsFilter[ 'prices' ][ 0 ] . '-' . $groupsFilter[ 'prices' ][ 1 ];
                } else {
                    $link = 'price-' . $groupsFilter[ 'prices' ][ 0 ] . '-' . $groupsFilter[ 'prices' ][ 1 ];
                }
            }
            return $link;
        }
        
        /**
         * Build link with custom price values
         *
         * @param $min
         * @param $max
         *
         * @return string
         */
        public function buildPrice($min, $max)
        {
            $groupFilter = $this->groupFilters;
            $currentMin = null;
            $currentMax = null;
            if (!empty($groupFilter[ 'prices' ])) {
                $currentMin = $groupFilter[ 'prices' ][ 0 ];
                $currentMax = $groupFilter[ 'prices' ][ 1 ];
            }
            $this->groupFilters[ 'prices' ][ 0 ] = $min;
            $this->groupFilters[ 'prices' ][ 1 ] = $max;
            $link = $this->buildLink();
            if (!empty($currentMin) && !empty($currentMax)) {
                $this->groupFilters[ 'prices' ][ 0 ] = $currentMin;
                $this->groupFilters[ 'prices' ][ 1 ] = $currentMax;
            } else {
                unset ($this->groupFilters[ 'prices' ]);
            }
            return $link;
        }
        
        protected function writeBrands(array $groups)
        {
            usort(
                $groups[ Brand::className() ],
                function ($a, $b) {
                    /**
                     * @var Brand $a
                     * @var Brand $b
                     */
                    if ($a->sort > $b->sort) {
                        return 1;
                    } elseif ($a->sort < $b->sort) {
                        return -1;
                    } else {
                        return strcmp($a->lang->title, $b->lang->title);
                    }
                }
            );
            return implode('_', ArrayHelper::getColumn($groups[ Brand::className() ], 'lang.alias.value'));
        }
        
        protected function writePOC(array $groups)
        {
            usort(
                $groups[ ProductOptionCompl::className() ],
                function ($a, $b) {
                    /**
                     * @var ProductOptionCompl $a
                     * @var ProductOptionCompl $b
                     */
                    if ($a->group->sort > $b->group->sort) {
                        return 1;
                    } elseif ($a->group->sort < $b->group->sort) {
                        return -1;
                    } else {
                        if ($a->group->id > $b->group->id) {
                            return 1;
                        } elseif ($a->group->id < $b->group->id) {
                            return -1;
                        } else {
                            if ($a->sort > $b->sort) {
                                return 1;
                            } elseif ($a->sort < $b->sort) {
                                return -1;
                            } else {
                                return strcasecmp($a->lang->value, $b->lang->value);
                            }
                        }
                    }
                }
            );
            return implode(
                '_',
                ArrayHelper::getColumn(
                    $groups[ ProductOptionCompl::className() ],
                    'lang.alias.value'
                )
            );
        }
        
        protected function writePOE(array $groups)
        {
            usort(
                $groups[ ProductOptionExcl::className() ],
                function ($a, $b) {
                    /**
                     * @var ProductOptionExcl $a
                     * @var ProductOptionExcl $b
                     */
                    if ($a->group->sort > $b->group->sort) {
                        return 1;
                    } elseif ($a->group->sort < $b->group->sort) {
                        return -1;
                    } else {
                        if ($a->group->id > $b->group->id) {
                            return 1;
                        } elseif ($a->group->id < $b->group->id) {
                            return -1;
                        } else {
                            if ($a->sort > $b->sort) {
                                return 1;
                            } elseif ($a->sort < $b->sort) {
                                return -1;
                            } else {
                                return strcasecmp($a->lang->value, $b->lang->value);
                            }
                        }
                    }
                }
            );
            return implode(
                '_',
                ArrayHelper::getColumn(
                    $groups[ ProductOptionExcl::className() ],
                    'lang.alias.value'
                )
            );
        }
        
        protected function writeVOC(array $groups)
        {
            usort(
                $groups[ VariantOptionCompl::className() ],
                function ($a, $b) {
                    /**
                     * @var VariantOptionCompl $a
                     * @var VariantOptionCompl $b
                     */
                    if ($a->group->sort > $b->group->sort) {
                        return 1;
                    } elseif ($a->group->sort < $b->group->sort) {
                        return -1;
                    } else {
                        if ($a->group->id > $b->group->id) {
                            return 1;
                        } elseif ($a->group->id < $b->group->id) {
                            return -1;
                        } else {
                            if ($a->sort > $b->sort) {
                                return 1;
                            } elseif ($a->sort < $b->sort) {
                                return -1;
                            } else {
                                return strcasecmp($a->lang->value, $b->lang->value);
                            }
                        }
                    }
                }
            );
            return implode(
                '_',
                ArrayHelper::getColumn(
                    $groups[ VariantOptionCompl::className() ],
                    'lang.alias.value'
                )
            );
        }
        
        protected function writeVOE(array $groups)
        {
            usort(
                $groups[ VariantOptionExcl::className() ],
                function ($a, $b) {
                    /**
                     * @var VariantOptionExcl $a
                     * @var VariantOptionExcl $b
                     */
                    if ($a->group->sort > $b->group->sort) {
                        return 1;
                    } elseif ($a->group->sort < $b->group->sort) {
                        return -1;
                    } else {
                        if ($a->group->id > $b->group->id) {
                            return 1;
                        } elseif ($a->group->id < $b->group->id) {
                            return -1;
                        } else {
                            if ($a->sort > $b->sort) {
                                return 1;
                            } elseif ($a->sort < $b->sort) {
                                return -1;
                            } else {
                                return strcasecmp($a->lang->value, $b->lang->value);
                            }
                        }
                    }
                }
            );
            return implode(
                '_',
                ArrayHelper::getColumn(
                    $groups[ VariantOptionExcl::className() ],
                    'lang.alias.value'
                )
            );
        }
        
        protected function getPrices(string $filter)
        {
            $filters = $this->parseFilter($filter);
            foreach ($filters as $item) {
                if (preg_match('/^price-(\d+)-(\d+)$/', $item, $matches)) {
                    $this->groupFilters[ 'prices' ] = [
                        $matches[ 1 ],
                        $matches[ 2 ],
                    ];
                }
            }
        }
        
        public function has(string $alias): bool
        {
            $aliases = ArrayHelper::getColumn($this->getActiveAliases(), 'value');
            return in_array($alias, $aliases);
        }
    }