<?php
    
    namespace artbox\core\controllers;
    
    use Yii;
    use artbox\core\models\PageCategory;
    use artbox\core\models\PageCategorySearch;
    use yii\web\Controller;
    use yii\web\NotFoundHttpException;
    use yii\filters\VerbFilter;
    use yii\web\Response;
    use yii\filters\AccessControl;

    /**
     * PageCategoryController implements the CRUD actions for PageCategory model.
     */
    class PageCategoryController extends Controller
    {
        /**
         * @inheritdoc
         */
        public function behaviors()
        {
            return [
                'verbs' => [
                    'class'   => VerbFilter::className(),
                    'actions' => [
                        'delete' => [ 'POST' ],
                    ],
                ],
                'access' => [
                    'class' => AccessControl::className(),
                    'rules' => [
                        [
                            'actions' => [
                                'login',
                                'error',
                            ],
                            'allow'   => true,
                        ],
                        [
                            'allow' => true,
                            'roles' => [ '@' ],
                        ],
                    ],
                ],
            ];
        }
        
        /**
         * @inheritdoc
         */
        public function getViewPath()
        {
            return '@artbox/core/views/page-category';
        }
        
        /**
         * Lists all PageCategory models.
         *
         * @return mixed
         */
        public function actionIndex()
        {
            $searchModel = new PageCategorySearch();
            $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
            
            return $this->render(
                'index',
                [
                    'searchModel'  => $searchModel,
                    'dataProvider' => $dataProvider,
                ]
            );
        }
        
        /**
         * Displays a single PageCategory model.
         *
         * @param integer $id
         *
         * @return mixed
         */
        public function actionView($id)
        {
            return $this->render(
                'view',
                [
                    'model' => $this->findModel($id),
                ]
            );
        }
        
        /**
         * Creates a new PageCategory model.
         * If creation is successful, the browser will be redirected to the 'view' page.
         *
         * @return mixed
         */
        public function actionCreate()
        {
            $model = new PageCategory();
            $model->generateLangs();
            
            if ($model->loadWithLangs(Yii::$app->request) && $model->saveWithLangs()) {
                return $this->redirect(
                    [
                        'view',
                        'id' => $model->id,
                    ]
                );
            } else {
                return $this->render(
                    'create',
                    [
                        'model'      => $model,
                        'modelLangs' => $model->modelLangs,
                    ]
                );
            }
        }
        
        /**
         * Updates an existing PageCategory model.
         * If update is successful, the browser will be redirected to the 'view' page.
         *
         * @param integer $id
         *
         * @return mixed
         */
        public function actionUpdate($id)
        {
            $model = $this->findModel($id);
            $model->generateLangs();
            
            if ($model->loadWithLangs(Yii::$app->request) && $model->saveWithLangs()) {
                return $this->redirect(
                    [
                        'view',
                        'id' => $model->id,
                    ]
                );
            } else {
                return $this->render(
                    'update',
                    [
                        'model'      => $model,
                        'modelLangs' => $model->modelLangs,
                    ]
                );
            }
        }
        
        /**
         * Deletes an existing PageCategory model.
         * If deletion is successful, the browser will be redirected to the 'index' page.
         *
         * @param integer $id
         *
         * @return mixed
         */
        public function actionDelete($id)
        {
            $this->findModel($id)
                 ->delete();
            
            return $this->redirect([ 'index' ]);
        }
        
        /**
         * Finds the PageCategory model based on its primary key value.
         * If the model is not found, a 404 HTTP exception will be thrown.
         *
         * @param integer $id
         *
         * @return PageCategory the loaded model
         * @throws NotFoundHttpException if the model cannot be found
         */
        protected function findModel($id)
        {
            if (( $model = PageCategory::findOne($id) ) !== null) {
                return $model;
            } else {
                throw new NotFoundHttpException('The requested page does not exist.');
            }
        }
    
        /**
         * Returnes an array of matched categories,
         * response must be in format:
         * $out = [
         *      'results' => [
         *          [
         *              'id'   => 1,
         *              'text' => 'First',
         *          ],
         *          [
         *              'id' => 2,
         *              'text' => 'Second'
         *          ],
         *      ],
         *  ];
         * ! array key must be 'id' and 'text' !
         *
         * @param null $q
         * @return array
         */
        public function actionList($q = null)
        {
            \Yii::$app->response->format = Response::FORMAT_JSON;
            $out = [
                'results' => [
                    [
                        'id'   => '',
                        'text' => '',
                    ],
                ],
            ];
            if (!is_null($q)) {
                $categories = PageCategory::find()
                                      ->joinWith('lang')
                                      ->select(
                                          [
                                              'id',
                                              'text' => 'page_category_lang.title',
                                          ]
                                      )
                                      ->andFilterWhere(
                                          [
                                              'ilike',
                                              'page_category_lang.title',
                                              $q,
                                          ]
                                      )
                                      ->limit(20)
                                      ->asArray()
                                      ->all();
            
                if (!empty($categories)) {
                    $out[ 'results' ] = $categories;
                }
            }
        
            return $out;
        }
    }
